from collections.abc import Callable
from dataclasses import dataclass
from re import Pattern
from typing import Any

from qtpy.QtCore import (  # type: ignore
    QAbstractTableModel,
    QModelIndex,
    QObject,
    QPersistentModelIndex,
    Qt,
)
from qtpy.QtGui import QColor

from ..models import Image
from ._consensus_image_list import ConsensusImageList


class ImageTableModel(QAbstractTableModel):
    @dataclass(frozen=True)
    class Column:
        header: str | None = None
        selector: Callable[[Image], Any] | None = None
        validator: Callable[[Image], bool] | None = None

    def __init__(
        self, images: ConsensusImageList, parent: QObject | None = None
    ) -> None:
        super().__init__(parent)
        self._images = images
        self._posix_path_pattern: Pattern[str] | None = None
        self._posix_path_column = ImageTableModel.Column(
            header="Image",
            selector=lambda img: img.posix_path,
            validator=lambda img: (
                (
                    bool(self._posix_path_pattern.fullmatch(img.posix_path))
                    if self._posix_path_pattern is not None
                    else True
                )
                and not any(
                    other_img.posix_path == img.posix_path
                    for other_img in images
                    if other_img is not img
                )
            ),
        )
        self._columns = [
            self._posix_path_column,
            ImageTableModel.Column(
                header="Data type",
                selector=lambda img: img.dtype,
                validator=lambda img: img.dtype == images.consensus.dtype,
            ),
            ImageTableModel.Column(
                header="Scenes",
                selector=lambda img: img.n_scenes,
            ),
            ImageTableModel.Column(
                header="Timepoints",
                selector=lambda img: img.n_timepoints,
                validator=lambda img: (
                    img.is_timeseries == images.consensus.is_timeseries
                ),
            ),
            ImageTableModel.Column(
                header="Channels",
                selector=lambda img: img.n_channels,
                validator=lambda img: img.n_channels == images.consensus.n_channels,
            ),
            ImageTableModel.Column(
                header="Width [px]",
                selector=lambda img: img.size_x_px,
            ),
            ImageTableModel.Column(
                header="Height [px]",
                selector=lambda img: img.size_y_px,
            ),
            ImageTableModel.Column(
                header="Depth [px]",
                selector=lambda img: img.size_z_px,
                validator=lambda img: img.is_zstack == images.consensus.is_zstack,
            ),
            ImageTableModel.Column(
                header="Dimension order",
                selector=lambda img: img.dimension_order,
                validator=lambda img: (
                    img.dimension_order == images.consensus.dimension_order
                ),
            ),
            ImageTableModel.Column(
                header="Pixel size (X)",
                selector=lambda img: img.pixel_size_x_str or "unknown",
                validator=lambda img: (
                    img.pixel_size_x_str == images.consensus.pixel_size_x_str
                ),
            ),
            ImageTableModel.Column(
                header="Pixel size (Y)",
                selector=lambda img: img.pixel_size_y_str or "unknown",
                validator=lambda img: (
                    img.pixel_size_y_str == images.consensus.pixel_size_y_str
                ),
            ),
            ImageTableModel.Column(
                header="Pixel size (Z)",
                selector=lambda img: img.pixel_size_z_str or "unknown",
                validator=lambda img: (
                    img.pixel_size_z_str == images.consensus.pixel_size_z_str
                ),
            ),
            ImageTableModel.Column(
                header="Channel names",
                selector=lambda img: ", ".join(img.channel_names),
                validator=lambda img: (
                    tuple(img.channel_names) == tuple(images.consensus.channel_names)
                ),
            ),
        ]

    def rowCount(
        self, parent: QModelIndex | QPersistentModelIndex | None = None
    ) -> int:
        assert parent is None or not parent.isValid()
        return len(self._images)

    def columnCount(
        self, parent: QModelIndex | QPersistentModelIndex | None = None
    ) -> int:
        assert parent is None or not parent.isValid()
        return len(self._columns)

    def data(
        self,
        index: QModelIndex | QPersistentModelIndex,
        role: int = Qt.ItemDataRole.DisplayRole,
    ) -> Any:
        if (
            index.isValid()
            and 0 <= index.row() < len(self._images)
            and 0 <= index.column() < len(self._columns)
        ):
            img = self._images[index.row()]
            col = self._columns[index.column()]
            if role == Qt.ItemDataRole.DisplayRole and col.selector is not None:
                return col.selector(img)
            if role == Qt.ItemDataRole.BackgroundRole and col.validator is not None:
                return (
                    QColor(Qt.GlobalColor.white)
                    if col.validator(img)
                    else QColor(Qt.GlobalColor.red)
                )
            if (
                role == Qt.ItemDataRole.EditRole
                and index.column() == self._columns.index(self._posix_path_column)
            ):
                return img.posix_path
        return None

    def setData(
        self,
        index: QModelIndex | QPersistentModelIndex,
        value: Any,
        role: int = Qt.ItemDataRole.EditRole,
    ) -> bool:
        if (
            index.isValid()
            and 0 <= index.row() < len(self._images)
            and index.column() == self._columns.index(self._posix_path_column)
            and role == Qt.ItemDataRole.EditRole
        ):
            img = self._images[index.row()]
            img.posix_path = str(value)
            self.dataChanged.emit(index, index)
            return True
        return super().setData(index, value, role)

    def headerData(
        self,
        section: int,
        orientation: Qt.Orientation,
        role: int = Qt.ItemDataRole.DisplayRole,
    ) -> Any:
        if (
            0 <= section < len(self._columns)
            and orientation == Qt.Orientation.Horizontal
            and role == Qt.ItemDataRole.DisplayRole
        ):
            return self._columns[section].header
        return None

    def flags(self, index: QModelIndex | QPersistentModelIndex) -> Qt.ItemFlag:
        flags = super().flags(index)
        if index.isValid() and index.column() == self._columns.index(
            self._posix_path_column
        ):
            flags |= Qt.ItemFlag.ItemIsEditable
        return flags

    def set_posix_path_pattern(self, pattern: Pattern[str] | None) -> None:
        self._posix_path_pattern = pattern
        posix_path_col = self._columns.index(self._posix_path_column)
        self.dataChanged.emit(
            self.index(0, posix_path_col),
            self.index(self.rowCount() - 1, posix_path_col),
        )



from dataclasses import dataclass


@dataclass
class Image:
    orig_path: str
    posix_path: str
    dtype: str
    n_scenes: int
    n_timepoints: int
    n_channels: int
    size_z_px: int
    size_y_px: int
    size_x_px: int
    dimension_order: str
    pixel_size_x: float | None
    pixel_size_y: float | None
    pixel_size_z: float | None
    channel_names: list[str]

    @property
    def is_timeseries(self) -> bool:
        return self.n_timepoints > 1

    @property
    def is_zstack(self) -> bool:
        return self.size_z_px > 1

    @property
    def pixel_size_x_str(self) -> str | None:
        return f"{self.pixel_size_x:.6f}" if self.pixel_size_x is not None else None

    @property
    def pixel_size_y_str(self) -> str | None:
        return f"{self.pixel_size_y:.6f}" if self.pixel_size_y is not None else None

    @property
    def pixel_size_z_str(self) -> str | None:
        return f"{self.pixel_size_z:.6f}" if self.pixel_size_z is not None else None



from collections.abc import Iterable, MutableSequence
from dataclasses import dataclass
from typing import overload

from qtpy.QtCore import QAbstractItemModel, QModelIndex  # type: ignore

from ..models import Image


class ConsensusImageList(MutableSequence[Image]):
    @dataclass(frozen=True)
    class Consensus:
        dtype: str
        is_timeseries: bool
        is_zstack: bool
        n_channels: int
        dimension_order: str
        pixel_size_x_str: str | None
        pixel_size_y_str: str | None
        pixel_size_z_str: str | None
        channel_names: list[str]

    _DEFAULT_CONSENSUS = Consensus(
        dtype="uint16",
        is_timeseries=False,
        is_zstack=False,
        n_channels=1,
        dimension_order="TCZYX",
        pixel_size_x_str=None,
        pixel_size_y_str=None,
        pixel_size_z_str=None,
        channel_names=[],
    )

    def __init__(self, images: Iterable[Image] | None = None) -> None:
        super().__init__()
        self._model: QAbstractItemModel | None = None
        self._images: list[Image] = list(images) if images is not None else []
        self._consensus = self._DEFAULT_CONSENSUS
        self._update_consensus()

    @overload
    def __getitem__(self, key: int) -> Image:
        ...

    @overload
    def __getitem__(self, key: slice) -> "ConsensusImageList":
        ...

    def __getitem__(self, key):
        if isinstance(key, slice):
            raise NotImplementedError("Slicing not implemented")
        return self._images[key]

    @overload
    def __setitem__(self, key: int, value: Image) -> None:
        ...

    @overload
    def __setitem__(self, key: slice, value: Iterable[Image]) -> None:
        ...

    def __setitem__(self, key, value) -> None:
        if isinstance(key, slice):
            raise NotImplementedError("Slicing not implemented")
        self._images[key] = value
        if self._model is not None:
            self._model.dataChanged.emit(
                self._model.index(key, 0),
                self._model.index(key, self._model.columnCount() - 1),
            )
        self._update_consensus()

    @overload
    def __delitem__(self, key: int) -> None:
        ...

    @overload
    def __delitem__(self, key: slice) -> None:
        ...

    def __delitem__(self, key) -> None:
        if isinstance(key, slice):
            raise NotImplementedError("Slicing not implemented")
        if self._model is not None:
            self._model.beginRemoveRows(QModelIndex(), key, key)
        del self._images[key]
        if self._model is not None:
            self._model.endRemoveRows()
        self._update_consensus()

    def __len__(self) -> int:
        return len(self._images)

    def insert(self, index: int, value: Image) -> None:
        if self._model is not None:
            self._model.beginInsertRows(QModelIndex(), index, index)
        self._images.insert(index, value)
        if self._model is not None:
            self._model.endInsertRows()
        self._update_consensus()

    def set_model(self, model: QAbstractItemModel | None) -> None:
        self._model = model

    @property
    def consensus(self) -> "ConsensusImageList.Consensus":
        return self._consensus

    def _find_consensus(self, key):
        counts = {}
        for img in sorted(self._images, key=lambda img: img.posix_path):
            value = key(img)
            if isinstance(value, list):
                value = tuple(value)
            counts[value] = counts.get(value, 0) + 1
        if not counts:
            return None
        value = max(counts, key=lambda value: counts[value])
        if isinstance(value, tuple):
            return list(value)
        return value

    def _update_consensus(self) -> None:
        if self._images:
            new_consensus = ConsensusImageList.Consensus(
                dtype=self._find_consensus(lambda img: img.dtype),
                is_timeseries=self._find_consensus(lambda img: img.is_timeseries),
                is_zstack=self._find_consensus(lambda img: img.is_zstack),
                n_channels=self._find_consensus(lambda img: img.n_channels),
                dimension_order=self._find_consensus(lambda img: img.dimension_order),
                pixel_size_x_str=self._find_consensus(lambda img: img.pixel_size_x_str),
                pixel_size_y_str=self._find_consensus(lambda img: img.pixel_size_y_str),
                pixel_size_z_str=self._find_consensus(lambda img: img.pixel_size_z_str),
                channel_names=self._find_consensus(lambda img: img.channel_names),
            )
        else:
            new_consensus = self._DEFAULT_CONSENSUS
        if new_consensus != self._consensus:
            if self._model is not None:
                self._model.beginResetModel()
            self._consensus = new_consensus
            if self._model is not None:
                self._model.endResetModel()



import tarfile
from collections.abc import Generator, Sequence
from io import BytesIO
from pathlib import Path
from tarfile import TarInfo
from tempfile import TemporaryDirectory

import pandas as pd
from aicsimageio import AICSImage
from aicsimageio.exceptions import UnsupportedFileFormatError

from .models import Image


def load_images(path: str | Path, _base_path: Path | None = None) -> Sequence[Image]:
    images = []
    path = Path(path)
    if _base_path is None:
        _base_path = path.parent
        assert _base_path.is_dir()
    try:
        aics_img = AICSImage(path)
        img = Image(
            orig_path=str(path),
            posix_path=str(path.relative_to(_base_path).as_posix()),
            n_scenes=len(aics_img.scenes),
            n_timepoints=aics_img.dims.T if "T" in aics_img.dims.order else 1,
            n_channels=aics_img.dims.C if "C" in aics_img.dims.order else 1,
            size_z_px=aics_img.dims.Z if "Z" in aics_img.dims.order else 1,
            size_y_px=aics_img.dims.Y if "Y" in aics_img.dims.order else 1,
            size_x_px=aics_img.dims.X if "X" in aics_img.dims.order else 1,
            dtype=aics_img.dtype.name,
            dimension_order=aics_img.dims.order,
            channel_names=list(aics_img.channel_names),
            pixel_size_x=aics_img.physical_pixel_sizes[-1],
            pixel_size_y=aics_img.physical_pixel_sizes[-2],
            pixel_size_z=aics_img.physical_pixel_sizes[-3],
        )
        images.append(img)
    except UnsupportedFileFormatError:
        if path.is_dir():
            for subpath in sorted(path.iterdir()):
                images += load_images(subpath, _base_path=_base_path)
    return images


def can_load_images(path: str | Path) -> bool:
    path = Path(path)
    if path.is_dir():
        return True
    try:
        AICSImage.determine_reader(path)
    except UnsupportedFileFormatError:
        return False
    return True


def write_archive(
    archive_file: str | Path,
    images: Sequence[Image],
    ome_tiff: bool = False,
    compresslevel: int = 5,
) -> Generator[Image, None, None]:
    with tarfile.open(
        archive_file, mode="w:gz", compresslevel=compresslevel
    ) as tar_file:
        for img in images:
            if ome_tiff:
                aics_img = AICSImage(img.orig_path)
                with TemporaryDirectory() as temp_dir:
                    img_file = Path(temp_dir) / f"{Path(img.orig_path).stem}.tiff"
                    posix_path = str(Path(img.posix_path).with_suffix(".tiff"))
                    aics_img.save(img_file)
                    tar_file.add(img_file, arcname=posix_path)
            else:
                tar_file.add(img.orig_path, arcname=img.posix_path)
            yield img
        df = pd.DataFrame(
            data=[
                {
                    "image": (
                        img.posix_path
                        if not ome_tiff
                        else str(Path(img.posix_path).with_suffix(".tiff"))
                    ),
                    "dtype": img.dtype,
                    "n_scenes": img.n_scenes,
                    "n_timepoints": img.n_timepoints,
                    "n_channels": img.n_channels,
                    "size_z_px": img.size_z_px,
                    "size_y_px": img.size_y_px,
                    "size_x_px": img.size_x_px,
                    "dimension_order": img.dimension_order,
                    "pixel_size_x": img.pixel_size_x,
                    "pixel_size_y": img.pixel_size_y,
                    "pixel_size_z": img.pixel_size_z,
                    "channel_names": ",".join(img.channel_names),
                }
                for img in images
            ]
        )
        data = df.to_csv(index=False).encode()
        with BytesIO(data) as buf:
            tar_info = TarInfo(name="images.csv")
            tar_info.size = len(data)
            tar_file.addfile(tar_info, fileobj=buf)



import re
import sys
from collections.abc import Sequence
from pathlib import Path

from qtpy.QtCore import QObject, QSortFilterProxyModel, Qt, QThread, Signal
from qtpy.QtWidgets import (
    QApplication,
    QFileDialog,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMainWindow,
    QProgressBar,
    QPushButton,
    QSpinBox,
    QStatusBar,
    QVBoxLayout,
    QWidget,
)

from .models import Image
from .utils import write_archive
from .widgets import ConsensusImageList, ImageTableModel, ImageTableView


class MainWindow(QMainWindow):
    class ArchiveWriterThread(QThread):
        step = Signal(int, Image)
        completed = Signal()
        error = Signal(Exception)

        def __init__(
            self,
            archive_file: str | Path,
            images: Sequence[Image],
            ome_tiff: bool = False,
            compresslevel: int = 5,
            parent: QObject | None = None,
        ) -> None:
            super().__init__(parent)
            self._archive_file = archive_file
            self._images = images
            self._ome_tiff = ome_tiff
            self._compresslevel = compresslevel

        def run(self) -> None:
            try:
                archive_writer = write_archive(
                    self._archive_file,
                    self._images,
                    ome_tiff=self._ome_tiff,
                    compresslevel=self._compresslevel,
                )
                for i, img in enumerate(archive_writer):
                    self.step.emit(i, img)
                self.completed.emit()
            except Exception as e:
                self.error.emit(e)

    def __init__(self, parent: QWidget | None = None) -> None:
        super().__init__(parent)
        self._archive_writer_thread: MainWindow.ArchiveWriterThread | None = None
        central_widget = QWidget()
        central_widget_layout = QVBoxLayout()
        edit_widget = QWidget()
        edit_widget_layout = QHBoxLayout()
        self._regex_line_edit = QLineEdit()
        self._regex_line_edit.setPlaceholderText(
            "POSIX file path pattern (Python regular expression, optional)"
        )
        self._regex_line_edit.textChanged.connect(self._on_regex_line_edit_text_changed)
        edit_widget_layout.addWidget(self._regex_line_edit)
        edit_widget_layout.addStretch()
        self._remove_selected_rows_button = QPushButton("Remove selected rows")
        self._remove_selected_rows_button.clicked.connect(
            self._on_remove_selected_rows_button_clicked
        )
        edit_widget_layout.addWidget(self._remove_selected_rows_button)
        edit_widget.setLayout(edit_widget_layout)
        central_widget_layout.addWidget(edit_widget)
        self._images = ConsensusImageList()
        self._image_table_model = ImageTableModel(self._images)
        self._image_table_model.modelReset.connect(lambda: self._update_button_states())
        self._image_table_model.dataChanged.connect(
            lambda top_left, bottom_right: self._update_button_states()
        )
        self._images.set_model(self._image_table_model)
        self._image_table_view = ImageTableView(self._images)
        sorted_image_table_model = QSortFilterProxyModel()
        sorted_image_table_model.setSourceModel(self._image_table_model)
        self._image_table_view.setModel(sorted_image_table_model)
        self._image_table_view.selectionModel().selectionChanged.connect(
            lambda selected, deselected: self._update_button_states()
        )
        central_widget_layout.addWidget(self._image_table_view)
        actions_widget = QWidget()
        actions_widget_layout = QHBoxLayout()
        actions_widget_layout.setAlignment(Qt.AlignmentFlag.AlignRight)
        self._compresslevel_label = QLabel("Compression level (1=fastest, 9=smallest):")
        actions_widget_layout.addWidget(self._compresslevel_label)
        self._compresslevel_spin_box = QSpinBox()
        self._compresslevel_spin_box.setRange(1, 9)
        self._compresslevel_spin_box.setValue(5)
        actions_widget_layout.addWidget(self._compresslevel_spin_box)
        actions_widget_layout.addStretch()
        self._convert_and_archive_button = QPushButton("Convert to OME-TIFF && archive")
        self._convert_and_archive_button.clicked.connect(
            self._on_convert_and_archive_button_clicked
        )
        actions_widget_layout.addWidget(self._convert_and_archive_button)
        self._archive_only_button = QPushButton("Archive only")
        self._archive_only_button.clicked.connect(self._on_archive_only_button_clicked)
        actions_widget_layout.addWidget(self._archive_only_button)
        actions_widget.setLayout(actions_widget_layout)
        central_widget_layout.addWidget(actions_widget)
        central_widget.setLayout(central_widget_layout)
        self.setCentralWidget(central_widget)
        self._status_bar = QStatusBar()
        self._status_bar.showMessage("Ready")
        self._progress_bar = QProgressBar()
        self._progress_bar.setHidden(True)
        self._status_bar.addPermanentWidget(self._progress_bar)
        self.setStatusBar(self._status_bar)
        self.setGeometry(100, 100, 800, 600)
        self.setWindowTitle("bircher")
        self._update_button_states()

    def _on_regex_line_edit_text_changed(self, text: str) -> None:
        if text:
            try:
                posix_path_pattern = re.compile(text)
                self._regex_line_edit.setStyleSheet("background-color: white")
            except re.error:
                posix_path_pattern = None
                self._regex_line_edit.setStyleSheet("background-color: red")
        else:
            posix_path_pattern = None
            self._regex_line_edit.setStyleSheet("background-color: white")
        self._image_table_model.set_posix_path_pattern(posix_path_pattern)

    def _on_remove_selected_rows_button_clicked(self) -> None:
        for index in sorted(
            self._image_table_view.selectionModel().selectedRows(),
            key=lambda index: index.row(),
            reverse=True,
        ):
            self._images.pop(index.row())

    def _on_convert_and_archive_button_clicked(self) -> None:
        self._create_archive(ome_tiff=True)

    def _on_archive_only_button_clicked(self) -> None:
        self._create_archive()

    def _create_archive(self, ome_tiff: bool = False) -> None:
        archive_file, selected_filter = QFileDialog.getSaveFileName(
            parent=self,
            dir=str(Path.home() / "Untitled"),
            filter="Image archive (*.tar.gz)",
            selectedFilter="Image archive (*.tar.gz)",
        )
        if archive_file:
            self._archive_writer_thread = MainWindow.ArchiveWriterThread(
                archive_file,
                self._images,
                ome_tiff=ome_tiff,
                compresslevel=self._compresslevel_spin_box.value(),
            )
            self._update_button_states()

            @self._archive_writer_thread.step.connect
            def on_archive_writer_thread_step(i, img):
                self._progress_bar.setValue(i + 1)

            @self._archive_writer_thread.completed.connect
            def on_archive_writer_thread_completed():
                self._status_bar.showMessage("Images archived")
                self._progress_bar.setHidden(True)
                self._archive_writer_thread = None
                self._update_button_states()

            @self._archive_writer_thread.error.connect
            def on_archive_writer_thread_error(e):
                self._status_bar.showMessage(f"Error: {e}")
                self._progress_bar.setHidden(True)
                self._archive_writer_thread = None
                self._update_button_states()

            self._status_bar.showMessage("Archiving images...")
            self._progress_bar.setMaximum(len(self._images))
            self._progress_bar.setHidden(False)
            self._progress_bar.setValue(0)
            self._archive_writer_thread.start()

    def _update_button_states(self) -> None:
        self._remove_selected_rows_button.setEnabled(
            self._archive_writer_thread is None
            and self._image_table_view.selectionModel().hasSelection()
        )
        self._convert_and_archive_button.setEnabled(
            self._archive_writer_thread is None
            and len(self._images) > 0
            and len(set(img.posix_path for img in self._images)) == len(self._images)
        )
        self._archive_only_button.setEnabled(
            self._archive_writer_thread is None
            and len(self._images) > 0
            and len(set(img.posix_path for img in self._images)) == len(self._images)
        )


def main():
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    app.exec()



import logging
from pathlib import Path

from qtpy.QtCore import QMimeData, Qt  # type: ignore
from qtpy.QtGui import QDragEnterEvent, QDragMoveEvent, QDropEvent
from qtpy.QtWidgets import QTableView, QWidget

from ..utils import can_load_images, load_images
from ._consensus_image_list import ConsensusImageList

log = logging.getLogger(__name__)


class ImageTableView(QTableView):
    def __init__(
        self, images: ConsensusImageList, parent: QWidget | None = None
    ) -> None:
        super().__init__(parent)
        self._images = images
        self.setAcceptDrops(True)
        self.setSortingEnabled(True)
        self.setSelectionBehavior(QTableView.SelectionBehavior.SelectRows)

    def dragEnterEvent(self, event: QDragEnterEvent) -> None:
        if self._check_mime_data(event.mimeData()):
            event.setDropAction(Qt.DropAction.CopyAction)
            event.accept()

    def dragMoveEvent(self, event: QDragMoveEvent) -> None:
        if event.dropAction() == Qt.DropAction.CopyAction and self._check_mime_data(
            event.mimeData()
        ):
            event.accept()

    def dropEvent(self, event: QDropEvent) -> None:
        if event.dropAction() == Qt.DropAction.CopyAction and self._check_mime_data(
            event.mimeData(), check_files=True
        ):
            for url in event.mimeData().urls():
                img_file_or_dir = Path(url.toLocalFile())
                try:
                    self._images += load_images(img_file_or_dir)
                except Exception as e:
                    log.error(f"Failed to load image(s) from {img_file_or_dir}: {e}")
            event.accept()

    def _check_mime_data(self, mime_data: QMimeData, check_files: bool = False) -> bool:
        if not mime_data.hasUrls() or any(
            not url.isLocalFile() for url in mime_data.urls()
        ):
            return False
        if check_files and (
            any(not Path(url.toLocalFile()).exists() for url in mime_data.urls())
            or any(not can_load_images(url.toLocalFile()) for url in mime_data.urls())
        ):
            return False
        return True



from ._consensus_image_list import ConsensusImageList
from ._image_table_model import ImageTableModel
from ._image_table_view import ImageTableView

__all__ = ["ConsensusImageList", "ImageTableModel", "ImageTableView"]
